// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *
 *		Multiprocessing under VM_UNT - Virtual OS
 *		     Declaration of basic primitives
 *
 * The present file contains description of operations that provide
 * multiprocessing and synchronization under Dr. C.-Q. Young's
 * virtual operating system, which is running on Sequent Symmetry 
 * series computer.
 *
 ************************************************************************
 */

#ifndef _mult_proc_h
#define _mult_proc_h 1
#pragma interface

/*
 *------------------------------------------------------------------------
 *			       Semaphores
 */

class Semaphore
{
  void * _sem_ptr;			// Pointer to the internal structure
  char name[20];			// Semaphore name
  short value;				// Semaphore current value

public:
  Semaphore(const char * name, const int in_value);
  ~Semaphore(void) {}
  void operator -- (void);		// P-operation
  void operator ++ (void);		// V-operation
  int awaited(void);			// True if somebody is waiting on
					// the semaphore

				// This would make assignment of the Semaphore
				// illegal (because it's stupid).
  Semaphore(Semaphore& dummy);
  Semaphore& operator = (Semaphore& dummy);
};

/*
 *------------------------------------------------------------------------
 *			 Threads of CPU control
 */

#if 0
typedef void (*Thread_body)(void);	// Describes what the thread has
					// got to do
#endif					// Too bad, the compiler failed to take
#define Thread_body(X) void X(void)	// that, so we had to rely on CPP

#if 1
extern "C" void proc_switching(void);	// This is what to call to switch
					// to another thread
extern "C" {
				// Create a thread. Return 0 if we're
				// in the child thread, and 1 otherwise
				// (kind of like fork())
  int newproc(const char * name, const short priority);
  volatile void endproc(void);	// Terminate the thread
}

class Thread
{

public:
  Thread(const char * name, const short priority, Thread_body((*body)));
  ~Thread(void) {}			// No way to kill it
};
#else
				// Disregard the following, please
extern void proc_switching(void);	// This is what to call to switch
					// to another thread
extern void ManageThreads(void);	// This is what to call to get
					// into multiprocessing

typedef unsigned short TID;		// Thread ID - index of the TCB in TCT

class Thread
{
  TID thread_id;

public:
  Thread(const char * name, const short priority, Thread_body((*body)));
  ~Thread(void) {}			// No way to kill it
};
#endif

#endif

