// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *
 *		       Basic Queue Operations
 *
 ************************************************************************
 */

#include "sysqueues.h"
#pragma implementation "sysqueues.h"

#include "myenv.h"

/*
 *------------------------------------------------------------------------
 *			Constructing and destructing
 */

				// Create an empty BasicQueue
BasicQueue::BasicQueue(void)
	: no_elements(0),
          access_lock("mutual exclusion lock",1),
          queue_sema("count",0)
{
  head = tail = 0;
}


/*
 *------------------------------------------------------------------------
 *		        Basic Queue Operations
 *
 */

				// Get the element from the top of the
				// queue. Wait if nothing is available
QueueLink * BasicQueue::get_from_head(void)
{
  queue_sema--;				// Tell the intention to get, or
					// wait if nothing available
  access_lock--;			// Get access to queue control fields
  assure(no_elements > 0,"The Basic queue is empty - nothing to get");
  no_elements--;
  QueueLink& el = *head;
  assert( el.prev == 0 );		// Head el should not have any
					// predecessors
  if( el.next != 0 )
    el.next->prev = 0;
  if( tail == &el )
    tail = el.prev;
  head = el.next;
  el.next = el.prev = 0;
  access_lock++;			// Release the lock - done with update
  return &el;
}

				// Add an element to the end of the Queue
void BasicQueue::append(QueueLink& el)
{
  access_lock--;			// Get access to queue control fields
  if( head == 0 )
    head = &el;
  el.next = 0;
  el.prev = tail;
  if( tail != 0 )
    tail->next = &el;
  tail = &el;
  no_elements++;
  access_lock++;			// Release the lock - done with update
  queue_sema++;				// Tell everybody we just enqueued smth
}

				// Dequeue a given element from the queue
				// An element should already be in the
				// queue
void BasicQueue::dequeue(QueueLink& el)
{
  assure(no_elements > 0,"The Basic queue is empty - cannot dequeue");
  queue_sema--;				// Decrease the count. Always succeds
					// due to the condition above
  access_lock--;			// Get access to queue control fields
  no_elements--;
  if( el.prev != 0 )
    el.prev->next = el.next;
  if( el.next != 0 )
    el.next->prev = el.prev;
  if( tail == &el )
    tail = el.prev;
  if( head == &el )
    head = el.next;
  el.next = el.prev = 0;
  access_lock++;			// Release the lock - done with update
}


/*
 *------------------------------------------------------------------------
 *			Service Queue Operations
 */

				// Dump IDs of elements in the queue
void BasicQueue::dump_ids(void)
{
  access_lock--;			// Get access to queue control fields

  if( no_elements == 0 )
    message("\nQueue is empty\n");
  else
  {
    register QueueLink * el = head;
    assert( el != 0 );
    message("\nID's of queue elements: ");
    for(; el != 0; el = el->next)
      message("%d ",el->id);
    message("\n");
  }
  access_lock++;			// Release the lock
}

				// If an element with the ID in a queue,
				// delete it from the queue
				// Return TRUE if it was successful
int BasicQueue::purge_id(const int id)
{
  if( no_elements != 0 )
  {
    register QueueLink * el = head;
    assert( el != 0 );
    for(; el != 0; el = el->next)
      if( el->id == id )
      {
	dequeue(*el);
	return 1;
      }
  }
  return 0;
}


